#include "Signalbilder.h"

// Pinbelegung Stellpult //
// PIN D2  - Signalleitung 1
// PIN D3  - Signalleitung 2
// PIN D4  - Signalleitung 3
// PIN D5  - Signalleitung 4
// PIN D6  - Eingang 60kmh (Hl3b, Hl6b, Hl9b, Hl12b)
// PIN D7  - Eingang 80kmh
// PIN D8  - Eingang 100kmh (Hl2, Hl5, Hl8)
// PIN D9  - Eingang Zs1 Ersatzsignal
// PIN D10 - Eingang Zusatzsignale Zs2, Zs6, Zs8, Zs13
// PIN D11 - Eingang Ra12 Rangierfahrtsignal
// PIN D12 - Eingang
// PIN D13 - Eingang
// PIN A0  - Taster Hp0
// PIN A1  - Taster Hp1/Hl1
// PIN A2  - Taster Hp2/Hl3a/Hl6a/Hl9a/Hl12a
// PIN A3  - LED Hp0 (rot)
// PIN A4  - LED Hp1 (grün)
// PIN A5  - LED Hp2 (gelb)
// PIN A6  -
// PIN A7  -

const char in_hp0  = A0;
const char in_hp1  = A1;
const char in_hp2  = A2;
const char in_V60  = 6;
const char in_V80  = 7;
const char in_V100 = 8;
const char in_Zs1  = 9;
const char in_ZsX  = 10;
const char in_Ra12 = 11;

const char out_LED0 = A3;
const char out_LED1 = A4;
const char out_LED2 = A5;

const char out_Leitung1 = 2;
const char out_Leitung2 = 3;
const char out_Leitung3 = 4;
const char out_Leitung4 = 5;

const char c_SignalbildHp0  = 'Hp0';
const char c_SignalbildHp1  = 'Hp1';
const char c_SignalbildHp2  = 'Hp2';
const char c_SignalbildV60  = 'V60';
const char c_SignalbildV80  = 'V80';
const char c_SignalbildV100 = 'V100';
const char c_SignalbildZs1  = 'Zs1';
const char c_SignalbildRa12 = 'Ra12';

int i_StateHp1;
int i_StateHp0;
int i_StateHp2;
int i_StateV60;
int i_StateV80;
int i_StateV100;
int i_StateZs1;
int i_StateZsX;
int i_StateRa12;

char Signalbild;
boolean Zusatzsignal;


// ************************************************************** //
// Signalbilder setzen
// ************************************************************** //
void setHp0( ) {
  digitalWrite(out_LED0, HIGH);
  digitalWrite(out_LED1, LOW);
  digitalWrite(out_LED2, LOW);
  digitalWrite(out_Leitung1, LOW);
  digitalWrite(out_Leitung2, HIGH);
  digitalWrite(out_Leitung3, HIGH);
}

void setHp1( ) {
  digitalWrite(out_LED0, LOW);
  digitalWrite(out_LED1, HIGH);
  digitalWrite(out_LED2, LOW);
  digitalWrite(out_Leitung1, HIGH);
  digitalWrite(out_Leitung2, HIGH);
  digitalWrite(out_Leitung3, HIGH);
}

void setHp2( ) {
  digitalWrite(out_LED0, LOW);
  digitalWrite(out_LED1, LOW);
  digitalWrite(out_LED2, HIGH);
  digitalWrite(out_Leitung1, LOW);
  digitalWrite(out_Leitung2, LOW);
  digitalWrite(out_Leitung3, HIGH);
}

void setV60( ) {
  digitalWrite(out_LED0, LOW);
  digitalWrite(out_LED1, LOW);
  digitalWrite(out_LED2, HIGH);
  digitalWrite(out_Leitung1, LOW);
  digitalWrite(out_Leitung2, LOW);
  digitalWrite(out_Leitung3, LOW);
}

void setV80( ) {
  digitalWrite(out_LED0, LOW);
  digitalWrite(out_LED1, LOW);
  digitalWrite(out_LED2, HIGH);
  digitalWrite(out_Leitung1, HIGH);
  digitalWrite(out_Leitung2, LOW);
  digitalWrite(out_Leitung3, LOW);
}

void setV100( ) {
  digitalWrite(out_LED0, LOW);
  digitalWrite(out_LED1, LOW);
  digitalWrite(out_LED2, HIGH);
  digitalWrite(out_Leitung1, HIGH);
  digitalWrite(out_Leitung2, LOW);
  digitalWrite(out_Leitung3, HIGH);
}

void setRa12( ) {
  digitalWrite(out_LED0, LOW);
  digitalWrite(out_LED1, LOW);
  digitalWrite(out_LED2, HIGH);
  digitalWrite(out_Leitung1, LOW);
  digitalWrite(out_Leitung2, HIGH);
  digitalWrite(out_Leitung3, LOW);
}

void setZs1( ) {
  digitalWrite(out_LED0, LOW);
  digitalWrite(out_LED1, LOW);
  digitalWrite(out_LED2, HIGH);
  digitalWrite(out_Leitung1, HIGH);
  digitalWrite(out_Leitung2, HIGH);
  digitalWrite(out_Leitung3, LOW);
}

void setZsXOn( ) {
  digitalWrite(out_Leitung4, HIGH);
}

void setZsXOff( ) {
  digitalWrite(out_Leitung4, LOW);
}

// ************************************************************** //
void setup() {
  // ************************************************************** //

  // Eingänge für Taster setzen
  pinMode(in_hp1,  INPUT);
  pinMode(in_hp0,  INPUT);
  pinMode(in_hp2,  INPUT);
  pinMode(in_V60,  INPUT);
  pinMode(in_V80,  INPUT);
  pinMode(in_V100, INPUT);
  pinMode(in_Zs1,  INPUT);
  pinMode(in_ZsX,  INPUT);
  pinMode(in_Ra12, INPUT);

  // Pullup aktivieren, damit liegt immer HIGH an
  // Taster schließt gegen GND, so dass wir auf LOW reagieren
  pinMode(in_hp0,  INPUT_PULLUP);
  pinMode(in_hp1,  INPUT_PULLUP);
  pinMode(in_hp2,  INPUT_PULLUP);
  pinMode(in_V60,  INPUT_PULLUP);
  pinMode(in_V80,  INPUT_PULLUP);
  pinMode(in_V100, INPUT_PULLUP);
  pinMode(in_Zs1,  INPUT_PULLUP);
  pinMode(in_ZsX,  INPUT_PULLUP);
  pinMode(in_Ra12, INPUT_PULLUP);

  // Ausgänge aktivieren
  pinMode(out_LED0, OUTPUT);
  pinMode(out_LED1, OUTPUT);
  pinMode(out_LED2, OUTPUT);
  pinMode(out_Leitung1, OUTPUT);
  pinMode(out_Leitung2, OUTPUT);
  pinMode(out_Leitung3, OUTPUT);
  pinMode(out_Leitung4, OUTPUT);

  // Initialzustand = rot
  setHp0( );

  //  Serial.begin(9600);
  //  digitalWrite(in_hp0, LOW);
  //  digitalWrite(in_hp1, LOW);
  //  digitalWrite(in_hp2, LOW);
}

// ************************************************************** //
void loop() {

  // ************************************************************** //
  // Eingangssignal lesen
  // ************************************************************** //

  i_StateHp0  = digitalRead(in_hp0);
  i_StateHp1  = digitalRead(in_hp1);
  i_StateHp2  = digitalRead(in_hp2);
  i_StateV60  = digitalRead(in_V60);
  i_StateV80  = digitalRead(in_V80);
  i_StateV100 = digitalRead(in_V100);
  i_StateRa12 = digitalRead(in_Ra12);
  i_StateZs1  = digitalRead(in_Zs1);
  i_StateZsX  = digitalRead(in_ZsX);

  if ( i_StateHp0 == LOW ) {
    Signalbild = c_SignalbildHp0;
  }

  if ( i_StateHp1 == LOW ) {
    Signalbild = c_SignalbildHp1;
  }

  if ( i_StateHp2 == LOW ) {
    Signalbild = c_SignalbildHp2;
  }

  if ( i_StateV60 == LOW ) {
    Signalbild = c_SignalbildV60;
  }

  if ( i_StateV80 == LOW ) {
    Signalbild = c_SignalbildV80;
  }

  if ( i_StateV100 == LOW ) {
    Signalbild = c_SignalbildV100;
  }

  if ( i_StateZs1 == LOW ) {
    Signalbild = c_SignalbildZs1;
  }

  if ( i_StateZsX == LOW ) {
    Zusatzsignal = true;
  } else {
    Zusatzsignal = false;
  }

  if ( i_StateRa12 == LOW ) {
    Signalbild = c_SignalbildV60;
  }


  // ************************************************************** //
  // Signalbild setzen
  // ************************************************************** //

  // Hauptsignalbild
  switch ( Signalbild ) {
    case c_SignalbildHp0:
      setHp0( );
      setZsXOff( );
      break;
    case c_SignalbildHp1:
      setHp1( );
      break;
    case c_SignalbildHp2:
      setHp2( );
      break;
    case 'V60':
      setV60;
      break;
    case 'V80':
      setV80( );
      break;
    case 'V100':
      setV100( );
      break;
    case 'Ra12':
      setRa12( );
      setZsXOff( );
      break;
    default:
      break;
  }

  // Zusatzsignal nur ein/aus
  if ( Zusatzsignal == true ) {
    setZsXOn( );
  } else {
    setZsXOff( );
  }


}

